package net.minecraft.src;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Random;
import java.util.Stack;

import net.minecraft.client.Minecraft;

public class SAPI {
	public static class Dungeon {
		private static ArrayList<PairFloatDungeonLoot> loot = new ArrayList<PairFloatDungeonLoot>();
		private static ArrayList<PairFloatString> mobs = new ArrayList<PairFloatString>();
		
		public static void addDungeonLoot(DungeonLoot dl) {addDungeonLoot(dl,1f);}
		public static void addDungeonLoot(DungeonLoot dl, float commonness) {
			loot.add(new PairFloatDungeonLoot(commonness,dl));
		}
		
		public static void addMob(String mob) {addMob(mob,1f);}
		public static void addMob(String mob, float commonness) {
			mobs.add(new PairFloatString(commonness,mob));
		}
		
		private static float lootGetMaxRandom() {
			float ret = 0f;
			for (PairFloatDungeonLoot pair : loot) ret += pair.f;
			return ret;
		}
		private static DungeonLoot lootGetFromRandomFloat(float random) {
			for (PairFloatDungeonLoot pair : loot) {
				if (random < pair.f) return pair.dl;
				random -= pair.f;
			}
			return null;
		}
		public static DungeonLoot lootGet(Random rnd) {
			return lootGetFromRandomFloat(rnd.nextFloat()*lootGetMaxRandom());
		}
		
		private static float mobGetMaxRandom() {
			float ret = 0f;
			for (PairFloatString pair : mobs) ret += pair.f;
			return ret;
		}
		private static String mobGetFromRandomFloat(float random) {
			for (PairFloatString pair : mobs) {
				if (random < pair.f) return pair.s;
				random -= pair.f;
			}
			return null;
		}
		public static String mobGet(Random rnd) {
			return mobGetFromRandomFloat(rnd.nextFloat()*mobGetMaxRandom());
		}
	}
	public static class Achievements {
		private static ArrayList<AchievementPage> pages = new ArrayList<AchievementPage>();
		private static ArrayList<Integer> hidden = new ArrayList<Integer>();
		private static int current = 0;
		protected static AchievementPage defaultPage = new AchievementPage();
		
		public static void addPage(AchievementPage page) {
			pages.add(page);
		}
	    public static AchievementPage getPage(Achievement achievement) {
	        if (achievement == null) return null;
	        for (AchievementPage page : pages) if (page.list.contains(achievement.statId)) return page;
	        return defaultPage;
	    }
		
		public static void hide(Achievement... achievements) {
			for (Achievement a : achievements) hidden.add(a.statId);
		}
		public static void unhide(Achievement... achievements) {
			for (Achievement a : achievements) hidden.remove((Object)a.statId);
		}
		public static boolean isHidden(Achievement achievement) {
			for (int a : hidden) if (achievement.statId == a) return true;
			return false;
		}
		
		public static ArrayList<AchievementPage> getPages() {return new ArrayList<AchievementPage>(pages);}
		public static AchievementPage current() {return pages.get(current);}
		public static void setPage(AchievementPage page) {
			for (int i = 0; i < pages.size(); i++) {
				if (pages.get(i).equals(page)) {current = i; return;}
			}
		}
	}
	public static class LockDifficulty {
		private static ArrayList<ILockDifficulty> mods = new ArrayList<ILockDifficulty>();
		
		public static void add(ILockDifficulty mod) {
			mods.add(mod);
		}
		public static String listLockingMods() {
	        String s = "";
	        if (mods.isEmpty()) return s;
	        for (ILockDifficulty mod : mods) {
	        	if (!mod.iLockShouldLock()) continue;
	            if (!s.isEmpty()) s += ",";
	            s += mod.iLockModName();
	        }
	        return s;
	    }
	}
	public static class NBT {
		private static ArrayList<INBT> nbt = new ArrayList<INBT>();
		
		public static void add(INBT inbt) {
			nbt.add(inbt);
		}
		public static ArrayList<INBT> getList() {
			return new ArrayList<INBT>(nbt);
		}
	}
	public static class Dimension {
		private static ArrayList<DimensionBase> dimensions = new ArrayList<DimensionBase>();
		private static Stack<Integer> order = new Stack<Integer>();
		
		public static void add(DimensionBase dimension) {
			dimensions.add(dimension);
		}
		public static DimensionBase getByID(int id) {
			for (DimensionBase dim : dimensions) if (dim.id == id) return dim;
			return null;
		}
		
		public static Stack<Integer> getOrder() {
			Stack<Integer> ret = new Stack<Integer>();
			ret.addAll(order);
			return ret;
		}
		public static void clearOrder() {order.clear();}
		public static void addToOrder(int dimensionID) {order.push(dimensionID);}
		
		public static void usePortal(int dimensionID, boolean useTeleporters) {
			System.out.println("Toggling dimension!!");
			
			Minecraft game = ModLoader.getMinecraftInstance();
			int dimOld = game.thePlayer.dimension;
			int dimNew = dimensionID;
			if (dimOld == dimNew) dimNew = 0;
			
			ArrayList<Entity> en = new ArrayList<Entity>();
			ArrayList<Class<? extends Entity>> entities = new ArrayList<Class<? extends Entity>>();
			ArrayList<Loc> offsets = new ArrayList<Loc>();
			ArrayList<Float> yaw = new ArrayList<Float>(), pitch = new ArrayList<Float>();
			ArrayList<Boolean> riding = new ArrayList<Boolean>();
			ArrayList<NBTTagCompound> nbts = new ArrayList<NBTTagCompound>();
			
			Loc loc = new Loc(game.thePlayer.posX,game.thePlayer.posY,game.thePlayer.posZ);
			Entity check = game.thePlayer;
			while (check.ridingEntity != null) {
				check = check.ridingEntity;
				entities.add(check.getClass());
				offsets.add(loc.subtract(new Loc(check.posX,check.posY,check.posZ)));
				riding.add(true);
				yaw.add(check.rotationYaw); pitch.add(check.rotationPitch);
				en.add(check);
				
				NBTTagCompound nbt = new NBTTagCompound();
				check.writeToNBT(nbt);
				nbts.add(nbt);
			}
			check = game.thePlayer;
			while (check.riddenByEntity != null) {
				check = check.riddenByEntity;
				entities.add(check.getClass());
				offsets.add(loc.subtract(new Loc(check.posX,check.posY,check.posZ)));
				riding.add(false);
				yaw.add(check.rotationYaw); pitch.add(check.rotationPitch);
				en.add(check);
				
				NBTTagCompound nbt = new NBTTagCompound();
				check.writeToNBT(nbt);
				nbts.add(nbt);
			}
			
			for (Entity e : en) game.theWorld.setEntityDead(e);
			en.clear();
			
			if (dimNew != 0) addToOrder(dimNew);
			if (dimNew == 0 && !order.isEmpty()) dimNew = order.pop();
			if (dimOld == dimNew) dimNew = 0;
			DimensionBase dbOld = getByID(dimOld), dbNew = getByID(dimNew);
			
			loc = dbOld.getDistanceScale(loc,true);
			game.thePlayer.dimension = dimNew; usePortalMoveEntity(game.thePlayer,loc);
			
			loc = dbNew.getDistanceScale(loc,false);
			World world = new World(game.theWorld,dbNew.makeWorldProvider());
			game.changeWorld(world,(dimNew != 0 ? "Entering" : "Leaving")+" the "+(dimNew != 0 ? dbNew.name : dbOld.name),game.thePlayer);
			game.thePlayer.worldObj = game.theWorld; usePortalMoveEntity(game.thePlayer,loc);
			
			if (useTeleporters) {
				Teleporter teleporter = dbNew.makeTeleporter();
				if (teleporter == null) teleporter = dbOld.makeTeleporter();
				teleporter.placeInPortal(game.theWorld,game.thePlayer);
			}
			
			for (int i = 0; i < entities.size(); i++) {
				try {
					Entity e = (Entity)(entities.get(i).getDeclaredConstructor(World.class).newInstance(game.theWorld));
					e.readFromNBT(nbts.get(i));
					game.theWorld.entityJoinedWorld(e);
					en.add(e);
				} catch (Exception e) {en.add(null);}
			}
			
			loc = new Loc(game.thePlayer.posX,game.thePlayer.posY,game.thePlayer.posZ);
			Entity lastR = game.thePlayer, lastRB = game.thePlayer;
			for (int i = 0; i < en.size(); i++) {
				Entity e = en.get(i); if (e == null) continue;
				e.rotationYaw = yaw.get(i); e.rotationPitch = pitch.get(i);
				usePortalMoveEntity(e,loc.add(offsets.get(i)));
				if (riding.get(i)) {lastR.mountEntity(e); lastR = e;} else {e.mountEntity(lastRB); lastRB = e;}
			}
		}
		private static void usePortalMoveEntity(Entity e, Loc loc) {
			if (e == null) return;
			Minecraft game = ModLoader.getMinecraftInstance();
			e.setLocationAndAngles(loc.x,loc.y,loc.z,e.rotationYaw,e.rotationPitch);
			if (e.isEntityAlive()) game.theWorld.updateEntityWithOptionalForce(e,false);
		}
		
		public static int getRespawnDimension() {
			Minecraft game = ModLoader.getMinecraftInstance();
			if (game.theWorld.worldProvider.canRespawnHere()) return game.theWorld.worldProvider.getDimensionID();
			for (int i = order.size()-1; i >= 0; i--) {
				DimensionBase dim = getByID(order.get(i));
				if (dim.makeWorldProvider().canRespawnHere()) return dim.id;
			}
			return 0;
		}
		public static void respawn(boolean flag, int dimension) {
			Minecraft game = ModLoader.getMinecraftInstance();
			
			clearOrder();
	        if (!game.theWorld.multiplayerWorld && !game.theWorld.worldProvider.canRespawnHere()) usePortal(dimension,false);
	        ChunkCoordinates chunkcoordinates = null;
	        ChunkCoordinates chunkcoordinates1 = null;
	        boolean flag1 = true;
	        if (game.thePlayer != null && !flag) {
	            chunkcoordinates = game.thePlayer.getPlayerSpawnCoordinate();
	            if (chunkcoordinates != null) {
	                chunkcoordinates1 = EntityPlayer.verifyRespawnCoordinates(game.theWorld,chunkcoordinates);
	                if (chunkcoordinates1 == null) game.thePlayer.addChatMessage("tile.bed.notValid");
	            }
	        }
	        if (chunkcoordinates1 == null) {
	            chunkcoordinates1 = game.theWorld.getSpawnPoint();
	            flag1 = false;
	        }
	        IChunkProvider ichunkprovider = game.theWorld.getIChunkProvider();
	        if (ichunkprovider instanceof ChunkProviderLoadOrGenerate) {
	            ChunkProviderLoadOrGenerate chunkproviderloadorgenerate = (ChunkProviderLoadOrGenerate)ichunkprovider;
	            chunkproviderloadorgenerate.setCurrentChunkOver(chunkcoordinates1.posX >> 4, chunkcoordinates1.posZ >> 4);
	        }
	        game.theWorld.setSpawnLocation();
	        game.theWorld.updateEntityList();
	        int j = 0;
	        if (game.thePlayer != null) {
	            j = game.thePlayer.entityId;
	            game.theWorld.setEntityDead(game.thePlayer);
	        }
	        game.renderViewEntity = null;
	        game.thePlayer = (EntityPlayerSP)game.playerController.createPlayer(game.theWorld);
	        game.thePlayer.dimension = dimension;
	        game.renderViewEntity = game.thePlayer;
	        game.thePlayer.preparePlayerToSpawn();
	        if (flag1) {
	        	game.thePlayer.setPlayerSpawnCoordinate(chunkcoordinates);
	        	game.thePlayer.setLocationAndAngles((float)chunkcoordinates1.posX + 0.5F, (float)chunkcoordinates1.posY + 0.1F, (float)chunkcoordinates1.posZ + 0.5F, 0.0F, 0.0F);
	        }
	        game.playerController.flipPlayer(game.thePlayer);
	        game.theWorld.spawnPlayerWithLoadedChunks(game.thePlayer);
	        game.thePlayer.movementInput = new MovementInputFromOptions(game.gameSettings);
	        game.thePlayer.entityId = j;
	        game.thePlayer.func_6420_o();
	        game.playerController.func_6473_b(game.thePlayer);
	        
	        try {
	            Method method = (net.minecraft.client.Minecraft.class).getDeclaredMethod("d",java.lang.String.class);
	            method.setAccessible(true);
	            method.invoke(game,"Respawning");
	        } catch (Exception e) {
	        	try {
		        	Method method = (net.minecraft.client.Minecraft.class).getDeclaredMethod("preloadWorld",java.lang.String.class);
		            method.setAccessible(true);
		            method.invoke(game,"Respawning");
	        	} catch (Exception e2) {e2.printStackTrace();}
	        }
	        if (game.currentScreen instanceof GuiGameOver) game.displayGuiScreen(null);
	    }
	}
	public static class Reach {
		private static ArrayList<IReach> reach = new ArrayList<IReach>();
		
		public static void add(IReach ireach) {
			reach.add(ireach);
		}
		public static ArrayList<IReach> getList() {
			return new ArrayList<IReach>(reach);
		}
		public static float getReach(boolean entity) {
			ItemStack hand = ModLoader.getMinecraftInstance().thePlayer.getHeldItem();
			for (IReach r : reach) if (r.iReachMatches(entity,hand)) return r.iReachGet(entity,hand);
			return entity ? 3f : 4f;
		}
	}
	public static class BlockSet {
		private static ArrayList<IBlockSet> blockset = new ArrayList<IBlockSet>();
		
		public static void add(IBlockSet iblockset) {
			blockset.add(iblockset);
		}
		public static ArrayList<IBlockSet> getList() {
			return new ArrayList<IBlockSet>(blockset);
		}
		public static IBlockSet getIntercept(World world, int x, int y, int z, int blockID) {
			for (IBlockSet i : blockset) if (i.iBlockMatches(world,x,y,z,blockID)) return i;
			return null;
		}
	}
	public static class Harvest {
		private static ArrayList<IHarvest> harvest = new ArrayList<IHarvest>();
		
		public static void add(IHarvest iharvest) {
			harvest.add(iharvest);
		}
		public static ArrayList<IHarvest> getList() {
			return new ArrayList<IHarvest>(harvest);
		}
		public static boolean intercept(World world, EntityPlayer player, int x, int y, int z, int blockID, int metadata) {
			for (IHarvest i : harvest) if (i.iHarvest(world,player,x,y,z,blockID,metadata)) return true;
			return false;
		}
	}
	
	private static class PairFloatDungeonLoot {
		private float f; private DungeonLoot dl;
		public PairFloatDungeonLoot(float f, DungeonLoot dl) {
			this.f = f;
			this.dl = dl;
		}
	}
	private static class PairFloatString {
		private float f; private String s;
		public PairFloatString(float f, String s) {
			this.f = f;
			this.s = s;
		}
	}
}